"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenAPISchemaValidator = void 0;
const ajv_draft_04_1 = require("ajv-draft-04");
const ajv_formats_1 = require("ajv-formats");
// https://github.com/OAI/OpenAPI-Specification/blob/master/schemas/v3.0/schema.json
const openapi3Schema = require("./openapi.v3.schema.json");
// https://github.com/OAI/OpenAPI-Specification/blob/master/schemas/v3.1/schema.json with dynamic refs replaced due to AJV bug - https://github.com/ajv-validator/ajv/issues/1745
const openapi31Schema = require("./openapi.v3_1.modified.schema.json");
const _2020_1 = require("ajv/dist/2020");
class OpenAPISchemaValidator {
    constructor(opts) {
        const options = {
            allErrors: true,
            validateFormats: true,
            coerceTypes: false,
            useDefaults: false,
            // Strict enforcement is nice, but schema is controlled by this library and known to be valid
            strict: false,
        };
        if (!opts.validateApiSpec) {
            options.validateSchema = false;
        }
        const [ok, major, minor] = /^(\d+)\.(\d+).(\d+)?$/.exec(opts.version);
        if (!ok) {
            throw Error('Version missing from OpenAPI specification');
        }
        ;
        if (major !== '3' || minor !== '0' && minor !== '1') {
            throw new Error('OpenAPI v3.0 or v3.1 specification version is required');
        }
        let ajvInstance;
        let schema;
        if (minor === '0') {
            schema = openapi3Schema;
            ajvInstance = new ajv_draft_04_1.default(options);
        }
        else if (minor == '1') {
            schema = openapi31Schema;
            ajvInstance = new _2020_1.default(options);
            // Open API 3.1 has a custom "media-range" attribute defined in its schema, but the spec does not define it. "It's not really intended to be validated"
            // https://github.com/OAI/OpenAPI-Specification/issues/2714#issuecomment-923185689
            // Since the schema is non-normative (https://github.com/OAI/OpenAPI-Specification/pull/3355#issuecomment-1915695294) we will only validate that it's a string
            // as the spec states
            ajvInstance.addFormat('media-range', true);
        }
        (0, ajv_formats_1.default)(ajvInstance, ['email', 'regex', 'uri', 'uri-reference']);
        ajvInstance.addSchema(schema);
        this.validator = ajvInstance.compile(schema);
    }
    validate(openapiDoc) {
        const valid = this.validator(openapiDoc);
        if (!valid) {
            return { errors: this.validator.errors };
        }
        else {
            return { errors: [] };
        }
    }
}
exports.OpenAPISchemaValidator = OpenAPISchemaValidator;
//# sourceMappingURL=openapi.schema.validator.js.map